/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2008  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS version: 3.62a                                         *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : BSP.c
Purpose : BSP for AVR32B AP7000 NGW100
--------  END-OF-HEADER  ---------------------------------------------
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

/****** Addresses of sfrs and peripheral identifiers ****************/
#define AVR32B_GPIO_PIOA_BASE_ADDR       0xFFE02800
#define AVR32B_GPIO_PIOE_BASE_ADDR       0xFFE03800

#define PIO_PER_OFFS                     0x00  // GPIO Enable Register
#define PIO_OER_OFFS                     0x10  // Output Driver Enable Register 
#define PIO_SODR_OFFS                    0x30  // Output Value Set Register
#define PIO_CODR_OFFS                    0x34  // Output Value Clear Register
#define PIO_ODSR_OFFS                    0x38  // Output Value Status Register

#define PIOA_PER       (*(volatile unsigned int*) (AVR32B_GPIO_PIOA_BASE_ADDR + PIO_PER_OFFS))
#define PIOA_OER       (*(volatile unsigned int*) (AVR32B_GPIO_PIOA_BASE_ADDR + PIO_OER_OFFS))
#define PIOA_SODR      (*(volatile unsigned int*) (AVR32B_GPIO_PIOA_BASE_ADDR + PIO_SODR_OFFS))
#define PIOA_CODR      (*(volatile unsigned int*) (AVR32B_GPIO_PIOA_BASE_ADDR + PIO_CODR_OFFS))
#define PIOA_ODSR      (*(volatile unsigned int*) (AVR32B_GPIO_PIOA_BASE_ADDR + PIO_ODSR_OFFS))

#define PIOE_PER       (*(volatile unsigned int*) (AVR32B_GPIO_PIOE_BASE_ADDR + PIO_PER_OFFS))
#define PIOE_OER       (*(volatile unsigned int*) (AVR32B_GPIO_PIOE_BASE_ADDR + PIO_OER_OFFS))
#define PIOE_SODR      (*(volatile unsigned int*) (AVR32B_GPIO_PIOE_BASE_ADDR + PIO_SODR_OFFS))
#define PIOE_CODR      (*(volatile unsigned int*) (AVR32B_GPIO_PIOE_BASE_ADDR + PIO_CODR_OFFS))
#define PIOE_ODSR      (*(volatile unsigned int*) (AVR32B_GPIO_PIOE_BASE_ADDR + PIO_ODSR_OFFS))

/****** Target specific configuration *******************************/
#define LEDA_BIT                         (19)   // LED A on NGW100 eval board, active low, port pin PA19 
#define LEDB_BIT                         (19)   // LED B on NGW100 eval board, active low, port pin PE19 

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PIOA_PER  = (1 << LEDA_BIT);
  PIOA_OER  = (1 << LEDA_BIT);
  PIOA_SODR = (1 << LEDA_BIT);
  PIOE_PER  = (1 << LEDB_BIT);
  PIOE_OER  = (1 << LEDB_BIT);
  PIOE_SODR = (1 << LEDB_BIT);  
}

/*********************************************************************
*
*       LED switching routines
*       LEDs are switched on with low level on port lines
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    PIOA_CODR = (1 << LEDA_BIT);
  } else if (Index == 1) {
    PIOE_CODR = (1 << LEDB_BIT);      
  }
}

void BSP_ClrLED(int Index) {
  if (Index == 0) {
    PIOA_SODR = (1 << LEDA_BIT);
  } else if (Index == 1) {
    PIOE_SODR = (1 << LEDB_BIT);      
  }
}

void BSP_ToggleLED(int Index) {
  if (Index == 0) {
	if (PIOA_ODSR & (1 << LEDA_BIT)) {
      PIOA_CODR = (1 << LEDA_BIT);
    } else {
      PIOA_SODR = (1 << LEDA_BIT);
    }  
  } else if (Index == 1) {
    if (PIOE_ODSR & (1 << LEDB_BIT)) {
      PIOE_CODR = (1 << LEDB_BIT);    
    } else {     
      PIOE_SODR = (1 << LEDB_BIT);
    }
  } 
}

/****** EOF *********************************************************/


