
  .section  .exception, "ax", @progbits

  .equ AVR32_SR_M0_OFFSET,  22
  .equ AVR32_SR_M0_SIZE, 1 
  .equ AVR32_SR_M1_SIZE, 1
  .equ AVR32_SR_M2_SIZE, 1
  .equ AVR32_INTC_IPR0_INTLEV_OFFSET, 30
  //Priorities
   .equ INT0,          0
   .equ INT1,          1
   .equ INT2,          2
   .equ INT3,          3
   .equ AVR32_SR, 0
  
// Start of Exception Vector Table.

  // EVBA must be aligned with a power of two strictly greater than the EVBA-
  // relative offset of the last vector.
  .balign 0x200

  // Export symbol.
  .global _evba
  .type _evba, @function
_evba:

        .org  0x000
        // Unrecoverable Exception.
_handle_Unrecoverable_Exception:
        rjmp $

        .org  0x004
        // TLB Multiple Hit: UNUSED IN AVR32UC.
_handle_TLB_Multiple_Hit:
        rjmp $

        .org  0x008
        // Bus Error Data Fetch.
_handle_Bus_Error_Data_Fetch:
        rjmp $

        .org  0x00C
         // Bus Error Instruction Fetch.
_handle_Bus_Error_Instruction_Fetch:
        rjmp $

        .org  0x010
        // NMI.
_handle_NMI:
        rjmp $

        .org  0x014
        // Instruction Address.
_handle_Instruction_Address:
        rjmp $

        .org  0x018
        // ITLB Protection.
_handle_ITLB_Protection:
        rjmp $

        .org  0x01C
        // Breakpoint.
_handle_Breakpoint:
        rjmp $

        .org  0x020
        // Illegal Opcode.
_handle_Illegal_Opcode:
        rjmp $

        .org  0x024
        // Unimplemented Instruction.
_handle_Unimplemented_Instruction:
        rjmp $

        .org  0x028
        // Privilege Violation.
_handle_Privilege_Violation:
        rjmp $

        .org  0x02C
        // Floating-Point: UNUSED IN AVR32UC.
_handle_Floating_Point:
        rjmp $

        .org  0x030
        // Coprocessor Absent: UNUSED IN AVR32UC.
_handle_Coprocessor_Absent:
        rjmp $

        .org  0x034
        // Data Address (Read).
_handle_Data_Address_Read:
        rjmp $

        .org  0x038
        // Data Address (Write).
_handle_Data_Address_Write:
        rjmp $

        .org  0x03C
        // DTLB Protection (Read).
_handle_DTLB_Protection_Read:
        rjmp $

        .org  0x040
        // DTLB Protection (Write).
_handle_DTLB_Protection_Write:
        rjmp $

        .org  0x044
        // DTLB Modified: UNUSED IN AVR32UC.
_handle_DTLB_Modified:
        rjmp $

        .org  0x050
        // ITLB Miss: UNUSED IN AVR32UC.
_handle_ITLB_Miss:
        rjmp $

        .org  0x060
        // DTLB Miss (Read): UNUSED IN AVR32UC.
_handle_DTLB_Miss_Read:
        rjmp $

        .org  0x070
        // DTLB Miss (Write): UNUSED IN AVR32UC.
_handle_DTLB_Miss_Write:
        rjmp $

        .org  0x080
        // DTLB Miss (Write): UNUSED IN AVR32UC.
        rjmp $

        .org  0x100
        // Supervisor Call.
_handle_Supervisor_Call:
        rjmp $


// Interrupt support.
// The interrupt controller must provide the offset address relative to EVBA.
// Important note:
//   All interrupts call a C function named _OS_Unhandled_Interrupt.
//   This function will read group and interrupt line number to then return in
//   R12 a pointer to a user-provided interrupt handler.

  .balign 4

_int0:
  nop                                  // For a yet unknown reason, this nop is required. SK/RS
  pushm     R11                        // Dummy. Used to store the PC for the final jump to the handler
  pushm     R8-R12, LR
  mov       r12, 0                     // Pass the int_lev parameter to the _get_interrupt_handler function.
IntHandler:
  rcall     OS_GetIRQHandler           // Return pointer to actual handler
  cp.w      r12, 0 
  breq      EndSpuriousInt
  //
  // Replace dummy with handler addr. and return via pop PC since we can not modify any registers.
  //
  frs                                  // Return stack needs to be invalidated since we need to jump via a pop PC
  st.w      R13[4*6],R12
  popm      R8-R12, LR
  popm      pc

EndSpuriousInt:
  popm     R8-R12, LR
  popm     R11                         // Dummy
  rete

  .balign 4
_int1:
  nop                                  // For a yet unknown reason, this nop is required. SK/RS
  pushm     R11                        // Dummy. Used to store the PC for the final jump to the handler
  pushm     R8-R12, LR
  mov       r12, 1                     // Pass the int_lev parameter to the _get_interrupt_handler function.
  bral      IntHandler

  .balign 4
_int2:
  nop                                  // For a yet unknown reason, this nop is required. SK/RS
  pushm     R11                        // Dummy. Used to store the PC for the final jump to the handler
  pushm     R8-R12, LR
  mov       r12, 2                     // Pass the int_lev parameter to the _get_interrupt_handler function.
  bral      IntHandler

  .balign 4
_int3:
  nop                                  // For a yet unknown reason, this nop is required. SK/RS
  pushm     R11                        // Dummy. Used to store the PC for the final jump to the handler
  pushm     R8-R12, LR
  mov       r12, 3                     // Pass the int_lev parameter to the _get_interrupt_handler function.
  bral      IntHandler


// Constant data area.

  .balign 4

  .equ AVR32_INTC_IPR0_INTLEV_OFFSET, 30

  // Values to store in the interrupt priority registers for the various interrupt priority levels.
  // The interrupt priority registers contain the interrupt priority level and
  // the EVBA-relative interrupt vector offset.
  .global ipr_val
  .type ipr_val, @object
ipr_val:
  .word (INT0 << AVR32_INTC_IPR0_INTLEV_OFFSET) | (_int0 - _evba)
  .word (INT1 << AVR32_INTC_IPR0_INTLEV_OFFSET) | (_int1 - _evba)
  .word (INT2 << AVR32_INTC_IPR0_INTLEV_OFFSET) | (_int2 - _evba)
  .word (INT3 << AVR32_INTC_IPR0_INTLEV_OFFSET) | (_int3 - _evba)
 

  

//! \endverbatim
//! @}
